import os

from py_auto_starter import auto_starter

if __name__ == '__main__':
    name = 'test'
    path = os.path.abspath('test.exe')

    assert not auto_starter.exists(name)

    auto_starter.add(name, path)
    assert auto_starter.exists(name)

    auto_starter.remove(name)
    assert not auto_starter.exists(name)



import winreg


class WinAutoStarter:

    def _get_key(self):
        return winreg.OpenKey(winreg.HKEY_CURRENT_USER, r'Software\Microsoft\Windows\CurrentVersion\Run',
                              0, winreg.KEY_ALL_ACCESS)

    def add(self, name, app_path):
        """add a new autostart entry"""
        key = self._get_key()
        try:
            winreg.SetValueEx(key, name, 0, winreg.REG_SZ, app_path)
        finally:
            winreg.CloseKey(key)

    def exists(self, name):
        """check if an autostart entry exists"""
        key = self._get_key()
        try:
            winreg.QueryValueEx(key, name)
            return True
        except FileNotFoundError:
            return False
        finally:
            winreg.CloseKey(key)

    def remove(self, name):
        """delete an autostart entry"""
        if self.exists(name):
            key = self._get_key()
            try:
                winreg.DeleteValue(key, name)
            finally:
                winreg.CloseKey(key)



import sys

if sys.platform.startswith('win32'):
    from .windows import WinAutoStarter as AutoStarter
elif sys.platform.startswith('linux'):
    from .linux import LinuxAutoStarter as AutoStarter
elif sys.platform.startswith('darwin'):
    from .mac import MacAutoStarter as AutoStarter
else:
    raise Exception('Unsupported platform')

auto_starter = AutoStarter()



from setuptools import setup
from setuptools import find_packages

VERSION = '0.0.1'
with open('README.md', encoding='utf-8') as f:
    LONG_DESCRIPTION = f.read()

setup(
    name='py-auto-starter',  # package name
    version=VERSION,  # package version
    description='cross-platform auto start',  # package description
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url='https://github.com/shi-hou/py-auto-starter',
    license='MIT',
    packages=find_packages(),
    platforms=['windows', 'macos', 'linux'],
    zip_safe=False,
    classifiers=[
        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',
    ],
)



import os


class LinuxAutoStarter:

    def __init__(self):
        _xdg_config_home = os.environ.get("XDG_CONFIG_HOME", "~/.config")
        self._xdg_user_autostart = os.path.join(os.path.expanduser(_xdg_config_home),
                                                "autostart")
        if not os.path.exists(self._xdg_user_autostart):
            os.makedirs(self._xdg_user_autostart)

    def _getfilename(self, name):
        """get the filename of an autostart (.desktop) file"""
        return os.path.join(self._xdg_user_autostart, name + ".desktop")

    def add(self, name, app_path):
        """add a new autostart entry"""
        desktop_entry = "[Desktop Entry]\n" \
                        "Name=%s\n" \
                        "Exec=%s\n" \
                        "Type=Application\n" \
                        "Terminal=false\n" % (name, app_path)
        with open(self._getfilename(name), "w") as f:
            f.write(desktop_entry)

    def exists(self, name):
        """check if an autostart entry exists"""
        return os.path.exists(self._getfilename(name))

    def remove(self, name):
        """delete an autostart entry"""
        if self.exists(name):
            os.unlink(self._getfilename(name))



import os
import plistlib


class MacAutoStarter:

    def __init__(self):
        self.launch_agents = '~/Library/LaunchAgents'
        if not os.path.exists(self.launch_agents):
            os.makedirs(self.launch_agents)

    def _getfilename(self, name):
        """get the filename of an autostart (.plist) file"""
        return os.path.join(self.launch_agents, name + ".plist")

    def add(self, name, app_path):
        """add a new autostart entry"""
        pl = dict(
            Label=name,
            RunAtLoad=True,
            Program=app_path
        )
        with open(self._getfilename(name), 'wb') as fp:
            plistlib.dump(pl, fp)

    def exists(self, name):
        """check if an autostart entry exists"""
        return os.path.exists(self._getfilename(name))

    def remove(self, name):
        """delete an autostart entry"""
        if self.exists(name):
            os.unlink(self._getfilename(name))
